-----------------------------------------------------------------------------------------------
-- Client Lua Script for BetterGroupWindow
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------
 
require "Window"
require "Apollo"
 
-----------------------------------------------------------------------------------------------
-- BetterGroupWindow Module Definition
-----------------------------------------------------------------------------------------------
local BetterGroupWindow = {} 
 
-----------------------------------------------------------------------------------------------
-- Constants
-----------------------------------------------------------------------------------------------
-- e.g. local kiExampleVariableMax = 999

local defaultSettings = {
	grpWinPosition = {0, 150, 300, 516},
	bLock = false,
	bHideMembers = false,
}

local userSettings = {
	grpWinPosition = defaultSettings.grpWinPosition,
	bLock = defaultSettings.bLock,
	bHideMembers = defaultSettings.bHideMembers,
}

-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function BetterGroupWindow:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 

    -- initialize variables here

    return o
end

function BetterGroupWindow:Init()
	local bHasConfigureFunction = false
	local strConfigureButtonText = ""
	local tDependencies = {
		"GroupDisplay",
	}
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
end
 

-----------------------------------------------------------------------------------------------
-- Save and Restore
-----------------------------------------------------------------------------------------------
function BetterGroupWindow:OnSave(eType)
	if eType ~= GameLib.CodeEnumAddonSaveLevel.Account then
		return
	end
		
	self:SaveWindowPosition(userSettings.grpWinPosition)
	
	return userSettings
	
end

function BetterGroupWindow:OnRestore(eType, tSave)
	if eType ~= GameLib.CodeEnumAddonSaveLevel.Account then
		return
	end
		
	if tSave then
		for k,v in pairs(tSave) do
			userSettings[k] = v
		end
	end
		
	if tSave.grpWinPosition then
		for k,v in pairs(tSave.grpWinPosition) do
			userSettings.grpWinPosition[k] = v
		end
	end
	
end

-----------------------------------------------------------------------------------------------
-- BetterGroupWindow OnLoad
-----------------------------------------------------------------------------------------------
function BetterGroupWindow:OnLoad()

	self:InitializeHooks()
	
	self.onRestoreCalled = false
	self.onDocLoadedCalled = false
	
    -- load our form file
	self.xmlDoc = XmlDoc.CreateFromFile("BetterGroupWindow.xml")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)
end

function BetterGroupWindow:InitializeHooks()
	crbGroupDisplay = Apollo.GetAddon("GroupFrame")
	
end

-----------------------------------------------------------------------------------------------
-- BetterGroupWindow OnDocLoaded
-----------------------------------------------------------------------------------------------
function BetterGroupWindow:OnDocLoaded()

	if self.xmlDoc ~= nil and self.xmlDoc:IsLoaded() then
	    self.wndSettings = Apollo.LoadForm(self.xmlDoc, "SettingsForm", nil, self)
		if self.wndSettings == nil then
			Apollo.AddAddonErrorText(self, "Could not load the main window for some reason.")
			return
		end
		
	    self.wndSettings:Show(false, true)

		-- if the xmlDoc is no longer needed, you should set it to nil
		-- self.xmlDoc = nil
		
		-- Register handlers for events, slash commands and timer, etc.
		-- e.g. Apollo.RegisterEventHandler("KeyDown", "OnKeyDown", self)
		Apollo.RegisterSlashCommand("bgw", "OnBetterGroupWindowOn", self)
		Apollo.RegisterSlashCommand("BGW", "OnBetterGroupWindowOn", self)


		-- Do additional Addon initialization here
		self.loadTimer = ApolloTimer.Create(0.100, true, "OnLoadTimer", self)
		
	end
end

-----------------------------------------------------------------------------------------------
-- BetterGroupWindow Functions
-----------------------------------------------------------------------------------------------
-- Define general functions here

-- on SlashCommand "/bgw"
function BetterGroupWindow:OnBetterGroupWindowOn()

	if crbGroupDisplay.wndGroupHud:IsShown() then
		self.wndSettings:FindChild("ShowGroupWindowBtn"):Enable(false)
	end	

	self.wndSettings:Invoke() -- show the window
end

function BetterGroupWindow:OnLoadTimer()
	self:SetupSettings()
end

function BetterGroupWindow:SaveWindowPosition(tPosition)
	tPosition[1], tPosition[2], tPosition[3], tPosition[4] = crbGroupDisplay.wndGroupHud:GetAnchorOffsets()
end

function BetterGroupWindow:LoadWindowPosition(tPosition)
	crbGroupDisplay.wndGroupHud:SetAnchorOffsets(tPosition[1], tPosition[2], tPosition[3], tPosition[4])
end

function BetterGroupWindow:SetLockWindow(bLockWindow)
	crbGroupDisplay.wndGroupHud:SetStyle("Moveable", not bLockWindow)
end

function BetterGroupWindow:SetupSettings()
	
	if crbGroupDisplay.wndGroupHud then
		self.loadTimer:Stop()
		
		self.wndSettings:FindChild("LockGroupWindow"):SetCheck(userSettings.bLock)
		self.wndSettings:FindChild("HideMembersBtn"):SetCheck(userSettings.bHideMembers)
		self:LoadWindowPosition(userSettings.grpWinPosition)
		self:SetLockWindow(userSettings.bLock)
		
		if userSettings.bHideMembers then
			self:SetGroupHudOpacity(100.0, 50.0)
		end
	end
	
end

function BetterGroupWindow:SetGroupHudOpacity(nOpacity, nRate)
	crbGroupDisplay.wndGroupHud:SetOpacity(0.01, 2.0)
	crbGroupDisplay.wndGroupHud:FindChild("GroupControlsBtn"):SetOpacity(nOpacity, nRate)
	crbGroupDisplay.wndGroupHud:FindChild("GroupNameOverButton"):SetOpacity(nOpacity, nRate)
	crbGroupDisplay.wndGroupHud:FindChild("GroupWrongInstance"):SetOpacity(nOpacity, nRate)
	crbGroupDisplay.wndGroupHud:FindChild("GroupBagBtn"):SetOpacity(nOpacity, nRate)
	crbGroupDisplay.wndGroupHud:FindChild("GroupHudMessage"):SetOpacity(nOpacity, nRate)
	crbGroupDisplay.wndGroupHud:FindChild("GroupHudLeaveDialog"):SetOpacity(nOpacity, nRate)
end

function BetterGroupWindow:RestoreGroupHudOpacity()
	crbGroupDisplay.wndGroupHud:SetOpacity(1.0, 2.0)
	crbGroupDisplay.wndGroupHud:FindChild("GroupControlsBtn"):SetOpacity(1.0, 50.0)
	crbGroupDisplay.wndGroupHud:FindChild("GroupNameOverButton"):SetOpacity(1.0, 50.0)
	crbGroupDisplay.wndGroupHud:FindChild("GroupWrongInstance"):SetOpacity(1.0, 50.0)
	crbGroupDisplay.wndGroupHud:FindChild("GroupBagBtn"):SetOpacity(1.0, 50.0)
	crbGroupDisplay.wndGroupHud:FindChild("GroupHudMessage"):SetOpacity(1.0, 50.0)
	crbGroupDisplay.wndGroupHud:FindChild("GroupHudLeaveDialog"):SetOpacity(1.0, 50.0)
end

---------------------------------------------------------------------------------------------------
-- SettingsForm Functions
---------------------------------------------------------------------------------------------------

function BetterGroupWindow:ShowGroupWindowOnCheck( wndHandler, wndControl, eMouseButton )
	if not crbGroupDisplay.wndGroupHud:IsShown() then
		crbGroupDisplay.wndGroupHud:Show(true, true)
	end
end

function BetterGroupWindow:ShowGroupWindowOnUncheck( wndHandler, wndControl, eMouseButton )
	if crbGroupDisplay.wndGroupHud:IsShown() then
		crbGroupDisplay.wndGroupHud:Show(false, true)
	end
end

function BetterGroupWindow:LockOnCheck( wndHandler, wndControl, eMouseButton )
	self:SetLockWindow(true)
	userSettings.bLock = true
end

function BetterGroupWindow:LockOnUncheck( wndHandler, wndControl, eMouseButton )
	self:SetLockWindow(false)
	userSettings.bLock = false
end

function BetterGroupWindow:OnClose( wndHandler, wndControl, eMouseButton )
	self.wndSettings:FindChild("ShowGroupWindowBtn"):SetCheck(false)
	if crbGroupDisplay.wndGroupHud:IsShown() then
		crbGroupDisplay.wndGroupHud:Show(false, true)
	end
	self:SaveWindowPosition(userSettings.grpWinPosition)
	self.wndSettings:Close()
end

function BetterGroupWindow:OnDefault( wndHandler, wndControl, eMouseButton )
	self:LoadWindowPosition(defaultSettings.grpWinPosition)
	self:SaveWindowPosition(userSettings.grpWinPosition)
end

function BetterGroupWindow:OnHideMembersCheck( wndHandler, wndControl, eMouseButton )
	crbGroupDisplay.wndGroupPortraitContainer:Show(false, true)
	self:SetGroupHudOpacity(100.0, 50.0)
	userSettings.bHideMembers = true
end

function BetterGroupWindow:OnHideMembersUncheck( wndHandler, wndControl, eMouseButton )
	crbGroupDisplay.wndGroupPortraitContainer:Show(true, true)
	self:RestoreGroupHudOpacity()
	userSettings.bHideMembers = false
end

-----------------------------------------------------------------------------------------------
-- BetterGroupWindow Instance
-----------------------------------------------------------------------------------------------
local BetterGroupWindowInst = BetterGroupWindow:new()
BetterGroupWindowInst:Init()
